;--------------------------------------------------------------------------------------------------
;----------------------------------------routines FM75---------------------------------------------
;-------------------------------By Lucien Dos Santos Farret(LuciDs)--------------------------------
;	11/08/12
;	Routines:			FM75_SetConfig
;						FM75_GetRawTemperature
;						FM75_ConvertRawTempToCelcius
;---Includes:
#include				"I2C_Routines.asm"
;---Constants:			
						FM75_I2CADDR	=	$90		

						FM75_REGADDR_TEMP   =   $00
						FM75_REGADDR_CONF   =   $01
						FM75_REGADDR_THYST  =   $02
						FM75_REGADDR_TOS    =   $03
						
						FM75_RESOLUTION		=	5
						FM75_FAULTTOLERANCE =	3
						FM75_OSPOLARITY		=	2
							FM75_OSPOLARITY_ACTIVE_LOW	=	0
							FM75_OSPOLARITY_ACTIVE_HIGH	=	1
						FM75_THERMOMODE     =	1
							FM75_THERMOMODE_COMPARATOR	=	0
							FM75_THERMOMODE_INTERRUPT	=	1
						FM75_OPERATION      =	0
							FM75_OPERATION_ON			=	0
							FM75_OPERATION_OFF			=   1
							
;//////////////////////////////////////////////////////////////////////////////////////////////////
;///////////////////////////////////	FM75_SetConfig			///////////////////////////////////
;	>	Setup the FM75 internal registers 
;	In:			cf:		set:	Power off the FM75
;						reset:	Power on  the FM75
;	Out:		cf:		set:    Error
;						reset: Ok
;	Destroys:	A,F,B,C
FM75_SetConfig:
						di
						push 		af
						call		I2C_SendStart
						ld			c,	 	FM75_I2CADDR | I2C_WRFLAG
						call		I2C_WriteByte
						jr			c,		FM75_SetConfig_Error
						ld			c,		FM75_REGADDR_CONF
						call		I2C_WriteByte
						pop			af
						ld			a,		( 3 << FM75_RESOLUTION | 0 << FM75_FAULTTOLERANCE | FM75_OSPOLARITY_ACTIVE_LOW << FM75_OSPOLARITY | FM75_THERMOMODE_INTERRUPT << FM75_THERMOMODE | FM75_OPERATION_ON << FM75_OPERATION ) >> 1
						rla
						ld			c,		a
						call		I2C_WriteByte
						call 		I2C_SendStop
						xor			a
						ret
FM75_SetConfig_Error:
						pop			af
						scf
						ret

;//////////////////////////////////////////////////////////////////////////////////////////////////
;///////////////////////////////////	FM75_GetRawTemperature	///////////////////////////////////
;	>	Get the raw Temperature from the FM75
;	In:			Nothing
;	Out:		HL:             The temperature in 2's complement, proportionnal to the temp in celcius
;				cf:		set:    Error
;						reset:  Ok
;	Destroys:	A,F,B,C,H,L
FM75_GetRawTemperature:
						di
						call		I2C_SendStart
						ld			c,		FM75_I2CADDR | I2C_WRFLAG 
						call		I2C_WriteByte
						ret			c
						ld			c,		FM75_REGADDR_TEMP
						call		I2C_WriteByte
						
						call		I2C_SendStart
						ld			c,		FM75_I2CADDR | I2C_RDFLAG
						call		I2C_WriteByte
						
						scf
						call		I2C_ReadByte
						ld			h,		c
						
						xor			a
						call		I2C_ReadByte
						ld			l,		c
						
						ld			b,		4
FM75_GetRawTemperature_Loop:					
						rr			h
						rr			l
						djnz		FM75_GetRawTemperature_Loop 
						
						ld			a,		h
						and			00001111b
						ld			h,		a
						
						call		I2C_SendStop
						xor			a
						ret
						
;//////////////////////////////////////////////////////////////////////////////////////////////////
;///////////////////////////////////	FM75_ConvertRawTempToCelcius	///////////////////////////
;	>	Convert the 2's complement temperature in hl into a floating point value in OP1
;	In:			HL
;	Out:		OP1
;	Destroys:	A lot !
FM75_ConvertRawTempToCelcius:
						bit 		3,		h
						jr			z,		FM75_ConvertRawTempToCelcius_PositiveTemp
						ld			a,h
						cpl
						and 		00000111b
						ld			h,a
						ld			a,l
						cpl
						ld			l,a
						inc			hl
						Bcall(_SetXXXXOP2)
						Bcall(_InvOP2S)
						jr			FM75_ConvertRawTempToCelcius_Calibrate
FM75_ConvertRawTempToCelcius_PositiveTemp:
						Bcall(_SetXXXXOP2)
FM75_ConvertRawTempToCelcius_Calibrate:
						
						Bcall(_OP2ToOP1)
;We know that +125 C <-> 111 1101 0000b = 2000
;So Temp = hl * 125 / 2000
; 125 -> OP2
						ld			hl,		125
						Bcall(_SetXXXXOP2)
;OP1 * OP2 -> OP1
						Bcall(_FPMult)
; 2000 -> OP2						
						ld			hl,		2000
						Bcall(_SetXXXXOP2)
;OP1 / OP2 -> OP1
						Bcall(_fpdiv)
						
						ret
						

						
						
						
						
						
						
						
						
						
						
						
						
						
						
						
						
						
						
						